from ._TranslatedMessage import TranslatedMessage


class CustomError(object):
    def __init__(
        self,     # type: CustomError
        message,  # type: TranslatedMessage
    ):
        # type: (...) -> None

        self.message = message
