import pyscripts

from mbcontextmessagehandler import MbContextMessageBus
from pos_api import PosService


def get_pos_service(
    pos_id,     # type: int
):
    # type: (...) -> PosService

    return PosService(
        message_bus=MbContextMessageBus(mbcontext=pyscripts.mbcontext),
        pos_id=pos_id,
    )
