class ConfigInfo(object):
    def __init__(
        self,
        database_path: str,
        component_name: str = "EventManager",
        max_worker_threads: int = 10,
        maintenance_interval: int = 30 * 60 * 1000,
        acked_events_ttl: int = 7,
        abandoned_events_ttl: int = 30,
    ):
        self.database_path = database_path
        self.component_name = component_name

        self.max_worker_threads = max_worker_threads
        self.maintenance_interval = maintenance_interval
        self.acked_events_ttl = acked_events_ttl
        self.abandoned_events_ttl = abandoned_events_ttl
