from messagebus import TokenCreator, TokenPriority


class BusTokens(object):
    TK_AUDITLOG_RESYNC = TokenCreator.create_token(
        priority=TokenPriority.high,
        message_group="14",
        number="1",
    )
    TK_SALE_ORDER_MODIFIED = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group="A",
        number="32",
    )
    TK_AUDITLOG_GET_OPERATORS_LOGOUT_SESSIONS = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group="A",
        number="33",
    )
