from dailygoals_services.model.repository import DailyGoalsRepository
from dailygoals_services.service import DailyGoalsService


class UpdateTotalSoldInteractor:
    def __init__(self, daily_goals_repository, daily_goals_service):
        # type: (DailyGoalsRepository, DailyGoalsService) -> None

        self.daily_goals_repository = daily_goals_repository
        self.daily_goals_service = daily_goals_service

    def update(self):
        # type: () -> None

        self.daily_goals_service.update_daily_goals()
