class LogEntry(object):
    def __init__(
            self,
            business_period,
            event_type,
            pos_id=-1,
            created_at_gmt=None,
            session_id=None,
            event_data='',
            order_id=None
    ):
        # type: (str, str, int, str, str, str, str) -> None
        self.pos_id = pos_id
        self.business_period = business_period
        self.event_type = event_type
        self.created_at_gmt = created_at_gmt
        self.session_id = session_id
        self.event_data = event_data
        self.order_id = order_id
